==> powerutils-0.1.4.tar.gz: lib/src/Log.dart <==
import 'dart:async';
import 'dart:io';

import 'package:powerutils/powerutils.dart';

import 'Map.dart';
import 'List.dart';

void FATAL(Object? m)  => Log.instance.fatal(m);
void ERROR(Object? m)  => Log.instance.error(m);
void WARN(Object? m)   => Log.instance.warn(m);
void INFO(Object? m)   => Log.instance.info(m);
void LOG(Object? m)    => Log.instance.info(m);
void DEBUG(Object? m)  => Log.instance.debug(m);
void TRACE(Object? m)  => Log.instance.trace(m);

enum LEVELS { FATAL,ERROR,WARN,INFO,DEBUG,TRACE }

extension LevelsExtension on LEVELS {
  String get name => this.toString().substring(this.toString().lastIndexOf('.'));

  int get intValue {
    switch(this) {
      case LEVELS.FATAL: return 1;
      case LEVELS.ERROR: return 2;
      case LEVELS.WARN: return 3;
      case LEVELS.INFO: return 4;
      case LEVELS.DEBUG: return 5;
      default: return 6;
    }
  }

  bool operator <(LEVELS other) => this.intValue < other.intValue;
  bool operator >(LEVELS other) => this.intValue > other.intValue;
  bool operator <=(LEVELS other) => this.intValue <= other.intValue;
  bool operator >=(LEVELS other) => this.intValue >= other.intValue;
}

class LogEntry {
  final LEVELS level;
  final String message;

  LogEntry(this.level,this.message);
}

//import 'package:flutter/foundation.dart';

class Log {
  static Log? _instance;
  static Log get instance {
    if(_instance == null)
      _instance = Log();

    return _instance!;
  }
  
  bool metadata;
	bool colorize;
  bool usePrint;  // use print method or stdout
	LEVELS level;

  final _controller = StreamController<LogEntry>.broadcast();

  Stream<LogEntry> get stream => _controller.stream;

  Log({this.level = LEVELS.TRACE, this.colorize = false, this.metadata = false, this.usePrint = true});
	
	LEVELS getLevelValue (String lvl) {
		switch(lvl) {
			case 'FATAL':
				return LEVELS.FATAL;
			case 'ERROR':
				return LEVELS.ERROR;
			case 'WARN':
				return LEVELS.WARN;
			case 'INFO':
				return LEVELS.INFO;
			case 'DEBUG':
				return LEVELS.DEBUG;
			case 'TRACE':
				return LEVELS.TRACE;
			default:
				return LEVELS.TRACE;
    }
  }

  _log(Object? m, String stringLevel, LEVELS level) {
    if(colorize)
      _logColor(m ?? 'null',stringLevel,level);
    else
      _logNoColor(m ?? 'null',stringLevel,level);
  }

	void trace (Object? m) {
    if(LEVELS.TRACE <= level)
		  _log(m, 'TRACE', LEVELS.TRACE);
  }

	void debug (Object? m) {
		if(LEVELS.DEBUG <= level)
      _log( m, 'DEBUG', LEVELS.DEBUG);
  }

	void info (Object? m) {
		if(LEVELS.INFO <= level)
      _log( m, 'INFO ', LEVELS.INFO);
  }

	void warn (Object? m) {
		if(LEVELS.WARN <= level)
      _log( m, 'WARN ', LEVELS.WARN);
  }

	void error (Object? m) {
		if(LEVELS.ERROR <= level)
      _log( m, 'ERROR', LEVELS.ERROR);
  }

	void fatal (Object? m) {
		if(LEVELS.FATAL <= level)
      _log( m, 'FATAL', LEVELS.FATAL);
  }

	void _logNoColor (Object m, String stringLevel, LEVELS level) {
    String meta = '';
    if(metadata)
      meta = timeStamp() + stringLevel[0] + ' ';
    
		_print(level,meta + (m is Map<String,dynamic> ? m.JsonPretty : ( m is List ? m.JsonPretty : m.toString())));
  }

  void _print(LEVELS level, String m) {
    if(usePrint)
      print(m);
    else {
      switch(level) {
        case LEVELS.FATAL:
        case LEVELS.ERROR:
          stderr.writeln(m);
          break;
        default:
          stdout.writeln(m);
      }
    }
    _controller.sink.add(LogEntry(level,m));
  }

  void _logColor (Object m, String stringLevel, LEVELS level) {
    String s = '';

    if(metadata) {
      s = '\u001b[1m'+'\u001b[37m';

      switch(level) {
        case LEVELS.ERROR:
        case LEVELS.FATAL:
          s += '\u001b[41m';
          break;
        case LEVELS.WARN:
          s += '\u001b[43m';	
          break; 
        default:
          break;               
      }

      s += timeStamp()+'\u001b[0m';

      if(m is Map<String,dynamic>) 
        s += stringLevel[0] + ' ';
      else if(m is List)
        s += stringLevel[0] + ' ';
      else {
        if(level == 'TRACE')
          s += '\u001b[90m${stringLevel[0]} ';
        else if(level == 'DEBUG')
          s += '\u001b[37m${stringLevel[0]} ';
        else
          s += '${stringLevel[0]} ';
      }
    }
    
    _print(level,s + (m is Map<String,dynamic> ? m.JsonPretty : ( m is List ? m.JsonPretty : m.toString())));

  }
/*
	void _logColor (m, level) {
		s = "\u001b[1m"+"\u001b[37m"

		switch level
			case 'ERROR','FATAL'
				s += "\u001b[41m"
			case 'WARN '
				s += "\u001b[43m"	

		s += timeStamp()+(Config.processes isnt 0 ? '' : process.pid+'-')+'\u001b[0m'

		if isString(m)			
			if level is 'TRACE'	
				s += "\u001b[90m#level[0]  #m "
			else level is 'DEBUG'
				s += "\u001b[37m#level[0]  #m "
			else
				s += "#level[0]  #m "
		else
			s += level[0] + ' ' + util.inspect m, {colors:yes}

		emit {
			name: 'eLog'
			data: s+"\n"
		}

		#fileWrite s+'\n'
		if ready
			file.write s+'\n'
  }
*/
	String timeStamp() {
		var t=DateTime.now();
	
		var M=t.month;
		var D=t.day;
		var H=t.hour;
		var m=t.minute;
		var S=t.second;
		var s=t.millisecond;
		var Y=t.year;

		return "[$Y/${ (M<10? '0$M' : M) }/${ (D<10? '0$D' : D) } ${ (H<10? '0$H' : H) }:${ (m<10? '0$m' : m) }:${ (S<10? '0$S' : S) },${ (s<100? (s<10? '00$s' : '0$s') : s) }]-";
  }
}

==> powerutils-0.1.5.tar.gz: lib/src/Log.dart <==
import 'dart:async';
import 'dart:io';

import 'package:powerutils/powerutils.dart';

import 'Map.dart';
import 'List.dart';

void FATAL(Object? m)  => Log.instance.fatal(m);
void ERROR(Object? m)  => Log.instance.error(m);
void WARN(Object? m)   => Log.instance.warn(m);
void INFO(Object? m)   => Log.instance.info(m);
void LOG(Object? m)    => Log.instance.info(m);
void DEBUG(Object? m)  => Log.instance.debug(m);
void TRACE(Object? m)  => Log.instance.trace(m);

enum LEVELS { FATAL,ERROR,WARN,INFO,DEBUG,TRACE }

extension LevelsExtension on LEVELS {
  String get name => this.toString().substring(this.toString().lastIndexOf('.'));

  int get intValue {
    switch(this) {
      case LEVELS.FATAL: return 1;
      case LEVELS.ERROR: return 2;
      case LEVELS.WARN: return 3;
      case LEVELS.INFO: return 4;
      case LEVELS.DEBUG: return 5;
      default: return 6;
    }
  }

  bool operator <(LEVELS other) => this.intValue < other.intValue;
  bool operator >(LEVELS other) => this.intValue > other.intValue;
  bool operator <=(LEVELS other) => this.intValue <= other.intValue;
  bool operator >=(LEVELS other) => this.intValue >= other.intValue;
}

class LogEntry {
  final LEVELS level;
  final String message;

  LogEntry(this.level,this.message);
}

//import 'package:flutter/foundation.dart';

class Log {
  static Log? _instance;
  static Log get instance {
    if(_instance == null)
      _instance = Log();

    return _instance!;
  }
  
  bool metadata;
	bool colorize;
  bool usePrint;  // use print method or stdout
	LEVELS level;

  final _controller = StreamController<LogEntry>();

  Stream<LogEntry> get stream => _controller.stream;

  Log({this.level = LEVELS.TRACE, this.colorize = false, this.metadata = false, this.usePrint = true});
	
	LEVELS getLevelValue (String lvl) {
		switch(lvl) {
			case 'FATAL':
				return LEVELS.FATAL;
			case 'ERROR':
				return LEVELS.ERROR;
			case 'WARN':
				return LEVELS.WARN;
			case 'INFO':
				return LEVELS.INFO;
			case 'DEBUG':
				return LEVELS.DEBUG;
			case 'TRACE':
				return LEVELS.TRACE;
			default:
				return LEVELS.TRACE;
    }
  }

  _log(Object? m, String stringLevel, LEVELS level) {
    if(colorize)
      _logColor(m ?? 'null',stringLevel,level);
    else
      _logNoColor(m ?? 'null',stringLevel,level);
  }

	void trace (Object? m) {
    if(LEVELS.TRACE <= level)
		  _log(m, 'TRACE', LEVELS.TRACE);
  }

	void debug (Object? m) {
		if(LEVELS.DEBUG <= level)
      _log( m, 'DEBUG', LEVELS.DEBUG);
  }

	void info (Object? m) {
		if(LEVELS.INFO <= level)
      _log( m, 'INFO ', LEVELS.INFO);
  }

	void warn (Object? m) {
		if(LEVELS.WARN <= level)
      _log( m, 'WARN ', LEVELS.WARN);
  }

	void error (Object? m) {
		if(LEVELS.ERROR <= level)
      _log( m, 'ERROR', LEVELS.ERROR);
  }

	void fatal (Object? m) {
		if(LEVELS.FATAL <= level)
      _log( m, 'FATAL', LEVELS.FATAL);
  }

	void _logNoColor (Object m, String stringLevel, LEVELS level) {
    String meta = '';
    if(metadata)
      meta = timeStamp() + stringLevel[0] + ' ';
    
		_print(level,meta + (m is Map<String,dynamic> ? m.JsonPretty : ( m is List ? m.JsonPretty : m.toString())));
  }

  void _print(LEVELS level, String m) {
    if(usePrint)
      print(m);
    else {
      switch(level) {
        case LEVELS.FATAL:
        case LEVELS.ERROR:
          stderr.writeln(m);
          break;
        default:
          stdout.writeln(m);
      }
    }
    _controller.sink.add(LogEntry(level,m));
  }

  void _logColor (Object m, String stringLevel, LEVELS level) {
    String s = '';

    if(metadata) {
      s = '\u001b[1m'+'\u001b[37m';

      switch(level) {
        case LEVELS.ERROR:
        case LEVELS.FATAL:
          s += '\u001b[41m';
          break;
        case LEVELS.WARN:
          s += '\u001b[43m';	
          break; 
        default:
          break;               
      }

      s += timeStamp()+'\u001b[0m';

      if(m is Map<String,dynamic>) 
        s += stringLevel[0] + ' ';
      else if(m is List)
        s += stringLevel[0] + ' ';
      else {
        if(level == 'TRACE')
          s += '\u001b[90m${stringLevel[0]} ';
        else if(level == 'DEBUG')
          s += '\u001b[37m${stringLevel[0]} ';
        else
          s += '${stringLevel[0]} ';
      }
    }
    
    _print(level,s + (m is Map<String,dynamic> ? m.JsonPretty : ( m is List ? m.JsonPretty : m.toString())));

  }
/*
	void _logColor (m, level) {
		s = "\u001b[1m"+"\u001b[37m"

		switch level
			case 'ERROR','FATAL'
				s += "\u001b[41m"
			case 'WARN '
				s += "\u001b[43m"	

		s += timeStamp()+(Config.processes isnt 0 ? '' : process.pid+'-')+'\u001b[0m'

		if isString(m)			
			if level is 'TRACE'	
				s += "\u001b[90m#level[0]  #m "
			else level is 'DEBUG'
				s += "\u001b[37m#level[0]  #m "
			else
				s += "#level[0]  #m "
		else
			s += level[0] + ' ' + util.inspect m, {colors:yes}

		emit {
			name: 'eLog'
			data: s+"\n"
		}

		#fileWrite s+'\n'
		if ready
			file.write s+'\n'
  }
*/
	String timeStamp() {
		var t=DateTime.now();
	
		var M=t.month;
		var D=t.day;
		var H=t.hour;
		var m=t.minute;
		var S=t.second;
		var s=t.millisecond;
		var Y=t.year;

		return "[$Y/${ (M<10? '0$M' : M) }/${ (D<10? '0$D' : D) } ${ (H<10? '0$H' : H) }:${ (m<10? '0$m' : m) }:${ (S<10? '0$S' : S) },${ (s<100? (s<10? '00$s' : '0$s') : s) }]-";
  }
}
